// JavaScript Document
// author: Broes.nl
// this code may freely be used, changed and redistributed
// when you're literally copying and using the code, please refer to the author

// JSLint directives
/*global AppleGlassButton, AppleInfoButton, LoadingSpinner, widget */

var GUI = (function () {

    // default private properties
    var selectedImage = 0, // wetteronline.de current
        selectedSize = "large",
        sizes = { large: { ratio: 1, optionIndex: 0 }, medium: { ratio: 0.75, optionIndex: 1 }, small: { ratio: 0.5, optionIndex: 2 } },
        updateAllowed = true;
    
    // private methods
    function setup() { // perform setup steps
        
        // initialize Done-button
        new AppleGlassButton(document.getElementById("doneButton"), "Done", function () {
            var front = document.getElementById("front"),
                back = document.getElementById("back");

            if (window.widget) {
                widget.prepareForTransition("ToFront"); // freezes the widget and prepares it for the flip back to the front
            }

            back.style.display = "none";    // hide the back
            front.style.display = "block";  // show the front

            if (window.widget) {
                setTimeout(function () { widget.performTransition(); }, 0); // and flip the widget back to the front
            }
            
            // wait for the transition to finish
            setTimeout(function () { selectLoop(selectedImage); }, 900);
        });
        
        // initialize i-button
        new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "black", "black", function () {
            var front = document.getElementById("front"),
                back = document.getElementById("back");

            if (window.widget) {
                widget.prepareForTransition("ToBack"); // freezes the widget so that you can change it without the user noticing
            }

            front.style.display = "none";   // hide the front
            back.style.display = "block";   // show the back

            if (window.widget) {
                setTimeout(function () { widget.performTransition(); }, 0); // and flip the widget over
            }
        });

        LoadingSpinner.init();
        BroesUpdater.init("updatepanel");
        
        // only perform when widget
        if (window.widget) {
            
            widget.onhide = function () {
                BuienradarImage.stopAnimation();
                document.getElementById('radarImage').style.display = "none";
                //console.log("NL Weerradar widget.onhide");
            };
            
            widget.onshow = function () {
                document.getElementById('radarImage').style.display = "block";
                selectLoop(selectedImage);
                if (updateAllowed) {
                    BroesUpdater.isItTimeToUpdate();
                }
                //console.log("NL Weerradar widget.onshow");
            };
            
            // get preferences
            var preferenceUpdate = widget.preferenceForKey("updateAllowed"),
                preferenceSize = widget.preferenceForKey("newSize"),
                preferenceSource = widget.preferenceForKey("sourceImage"),
                preferenceOldSize = widget.preferenceForKey("size");
            
            // execute preferences
            if (preferenceUpdate !== undefined) {
                updateAllowed = preferenceUpdate;
                document.getElementById("updatecheck").checked = updateAllowed;
            }
            
            if (preferenceSource !== undefined) {
                selectedImage = parseInt(preferenceSource, 10);
                var selector = document.getElementById("loopSelect"),
                    i = 0;
                    
                for (i = 0; i < selector.options.length; i++) {
                    if (parseInt(selector.options[i].value, 10) === selectedImage)
                    {
                        selector.selectedIndex = i;
                    }
                }
            }
            
            if (preferenceSize !== undefined) {
                selectedSize = preferenceSize;
                document.getElementById('sizeSelect').selectedIndex = parseInt(sizes[selectedSize].optionIndex, 10);
            }
            else if (preferenceOldSize !== undefined) {                
                if (!preferenceOldSize) {
                    selectedSize = "small";
                    document.getElementById('sizeSelect').selectedIndex = parseInt(sizes[selectedSize].optionIndex, 10);
                }
                widget.setPreferenceForKey(null, "size");
            }

        }
        
        // for debugging purposes
        // document.getElementById('radarImage').onload = function () {
        //     console.log("loaded: " + this.src);
        // };
    }
    
    function selectSize(sizeValue) {
        selectedSize = sizeValue;
        
        if(window.widget) {
            widget.setPreferenceForKey(sizeValue, "newSize");
        }
    }
    
    function resizeFromTo(fromWidth, fromHeight, toWidth, toHeight) {

        var sizeRatio = sizes[selectedSize].ratio,
            toActualWidth = parseInt(toWidth * sizeRatio, 10),
            toActualHeight = parseInt(toHeight * sizeRatio, 10),
            radarImage = document.getElementById('radarImage');

        radarImage.style.width = toActualWidth;
        radarImage.style.height = toActualHeight;

        if ((fromWidth !== toActualWidth) && (fromHeight !== toActualHeight)) {
            
            var widthMargin = 18, // shadows left & right
                heightMargin = 40, // shadows top & bottom + footer
                widthDifference = fromWidth - toActualWidth,
                heightDifference = fromHeight - toActualHeight,
                selectButtonWidth = 132;

            radarImage.style.display = "none";
            
            var selectButton = document.getElementById("buttons");
            selectButton.style.display = "none";
            selectButton.style.left = (selectedSize !== "small") ? ((toWidth + widthMargin) / 2 * sizeRatio - selectButtonWidth / 2) + "px" : "8px"; // center the dropdown select button
            
                        
            var animator = new AppleAnimator(500, 40, 0.0, 1.0, function (animation, animationValue, start, finish) {
                window.resizeTo(parseInt(fromWidth - widthDifference * animationValue + widthMargin, 10), parseInt(fromHeight - heightDifference * animationValue + heightMargin, 10));
                if (finish) {
                    radarImage.style.display = "block";
                    selectButton.style.display = "block";
                }
            });
            animator.start();            
        }
        
        document.getElementById('loopSelect').disabled = false;
    }
    
    function errorLoading() {
        document.getElementById("alertMessage").style.display = "block";                
        LoadingSpinner.stop();
        document.getElementById('loopSelect').disabled = false;
    }

    function openSite(url) {
        if(window.widget) {
            if (url) {
                widget.openURL(url);
            }
            else {
                widget.openURL("http://www.broes.nl");
            }
        }
    }
    
    function openSourceURL() {
        var sourceURL = [
            "http://www.wetteronline.de/radar/euroddlnw.htm", // 0
            "http://www.wetteronline.de/radar/euroddlnw.htm", // 1
            "http://www.wetteronline.de/radar/euroddlnw.htm", // 2
            "http://www.buienradar.nl/", // 3
            "http://www.buienradar.nl/", // 4
            "http://www.buienradar.nl/h.aspx?jaar=-3&soort=verwachting1x1", // 5
            "http://www.sat24.nl/Region.aspx?country=nl&sat=vis&type=last", // 6
            "http://www.sat24.nl/Region.aspx?country=nl&sat=vis&type=loop", // 7
            "http://www.sat24.nl/Region.aspx?country=nl&sat=ir&type=last", // 8
            "http://www.sat24.nl/Region.aspx?country=nl&sat=ir&type=loop", // 9
            "http://www.knmi.nl/actueel/", // 10
            "http://www.knmi.nl/actueel/", // 11
            "http://www.knmi.nl/actueel/", // 12
            "http://www.knmi.nl/actueel/", // 13
            "http://www.knmi.nl", // 14
            "http://www.buienradar.nl/sneeuw.aspx", // 15
            "http://www.knmi.nl/actueel/" // 16
            ];
        if (window.widget) {
            widget.openURL(sourceURL[selectedImage]);
        }
    }
    
    function updateCheckbox() {

        updateAllowed = document.getElementById("updatecheck").checked;
        
        if (window.widget) {
            widget.setPreferenceForKey(updateAllowed, "updateAllowed");
        }
        
        if (!updateAllowed) {
            document.getElementById("versioninfo").innerHTML = "Deze versie: " + BroesUpdater.currentVersion;
        }
        else
        {
            BroesUpdater.isItTimeToUpdate(true);
        }
    }
    
    function selectLoop(a) {
        BuienradarImage.stopAnimation();
        a = parseInt(a, 10);
        selectedImage = a;
        var selectText = document.getElementById("selectText");
        document.getElementById("alertMessage").style.display = "none";
        document.getElementById('loopSelect').disabled = true;
        LoadingSpinner.start();
        
        switch (a) {
            // Wetteronline
            case 0:
                WetteronlineImage.loadImage("mostRecent", resizeFromTo);
                selectText.innerHTML = "Huidig beeld";
                break;
            case 1:
                WetteronlineImage.loadImage("lastNinetyMinutes", resizeFromTo);
                selectText.innerHTML = "90 minuten";
                break;
            case 2:
                WetteronlineImage.loadImage("lastThreeHours", resizeFromTo);
                selectText.innerHTML = "3 uur";
                break;

            // Buienradar
            case 3:
                selectText.innerHTML = "Huidig beeld";
                BuienradarImage.loadImage("mostRecent", resizeFromTo);
                break;
            case 4:
                selectText.innerHTML = "60 minuten";
                BuienradarImage.loadImage("lastSixtyMinutes", resizeFromTo);
                break;
            case 5:
                selectText.innerHTML = "Verwachting";
                BuienradarImage.loadImage("twoHourPrediction", resizeFromTo);
                break;
            case 15:
                selectText.innerHTML = "Sneeuwradar";
                //BuienradarImage.loadSnowRadar(resizeFromTo);
                BuienradarImage.loadImage("snowRadar", resizeFromTo);
                break;
                        
            // Sat24
            case 6:
                selectText.innerHTML = "Zichtbaar licht";
                Sat24Image.loadImage("lastVis", resizeFromTo);
                break;
            case 7:
                selectText.innerHTML = "Zichtbaar licht";
                Sat24Image.loadImage("loopVis", resizeFromTo);
                break;
            case 8:
                selectText.innerHTML = "Infrarood";
                Sat24Image.loadImage("lastIR", resizeFromTo);
                break;
            case 9:
                selectText.innerHTML = "Infrarood";
                Sat24Image.loadImage("loopIR", resizeFromTo);
                break;

            // KNMI actueel
            case 10:
                selectText.innerHTML = "Temperatuur";
                KnmiImage.loadImage("temp", resizeFromTo);
                break;
            case 11:
                selectText.innerHTML = "Zicht";
                KnmiImage.loadImage("zicht", resizeFromTo);
                break;
            case 12:
                selectText.innerHTML = "Wind (m/sec)";
                KnmiImage.loadImage("windMsec", resizeFromTo);
                break;
            case 13:
                selectText.innerHTML = "Wind (beaufort)";
                KnmiImage.loadImage("windBeaufort", resizeFromTo);
                break;
            case 14:
                selectText.innerHTML = "Neerslagradar";
                KnmiImage.loadImage("neerslagradar", resizeFromTo);
                break;
            case 16:
                selectText.innerHTML = "Gevoelstemp.";
                KnmiImage.loadImage("windchill", resizeFromTo);
                break;
        }
        
        if (window.widget) {
            widget.setPreferenceForKey(a, "sourceImage");
        }
        
    }
    
    // public methods
    return {
        setup: setup,
        resizeFromTo: resizeFromTo,
        errorLoading: errorLoading,
        openSite: openSite,
        openSourceURL: openSourceURL,
        selectLoop: selectLoop,
        updateCheckbox: updateCheckbox,
        selectSize: selectSize
    };
}());

Number.prototype.toTwoDigitString = function () {
    return (this < 10) ? "0" + this.toString() : this.toString();
};

var BuienradarImage = (function () {
    // private properties
    var width, height;
    var loadingAttempts = 0;
    var snowAnimationTimer;
    
    // private methods
    function randomNumber() {
        var returnValue = "",
            i = 0;
        for (i = 0; i < 6; i++) {
            returnValue += Math.round(10 * Math.random());
        }
        return returnValue;
    }
    
    function urlQuery() {
        var currentDateTime = new Date();
        var currentMinutes = currentDateTime.getMinutes();
        currentDateTime.setTime(currentDateTime - ( 5 + (currentMinutes % 5) + (loadingAttempts * 5) ) * 60 * 1000); // only full increments of 5min
        var jaar = currentDateTime.getFullYear(); // 4-digit year
        var dag = currentDateTime.getDate().toTwoDigitString(); // 1..31
        var maand = (1 + currentDateTime.getMonth()).toTwoDigitString(); // 0..11, 0=January
        var uur = currentDateTime.getHours().toTwoDigitString(); // 0..23
        var minuut = currentDateTime.getMinutes().toTwoDigitString(); // 0..59

        var urlTimeString = "&jaar=" + jaar + "&maand=" + maand + "&dag=" + dag + "&minuut="+ minuut + "&uur=" + uur;
        return urlTimeString;
    }
    
    function sources(sourceName) {
        switch (sourceName) {
            case "mostRecent":
                return "http://api.buienradar.nl/image/1.0/radarmapnl/gif/?hist=1&forc=0&width=550&rndm=" + randomNumber();
            case "lastSixtyMinutes":
                return "http://api.buienradar.nl/image/1.0/radarmapnl/gif/?hist=12&forc=0&width=550&l=1&rndm=" + randomNumber();
            case "twoHourPrediction":
                return "http://api.buienradar.nl/image/1.0/radarmapnl/gif/?hist=0&forc=36&width=550&l=1&step=0&rndm=" + randomNumber();
            case "snowRadar":
                return "http://api.buienradar.nl/image/1.0/snowmapnl/gif/?width=550&rndm=" + randomNumber();
        }
    }
    
    function loadImage(sourceName, callback) {
        var newRadarImage = new Image();
        
        newRadarImage.onload = function () {
            width = newRadarImage.width;
            height = newRadarImage.height;
            if (width > 10 && height > 10) { // if the image is bigger than 10x10px
                LoadingSpinner.stop();
                loadingAttempts = 0;
                var radarImage = document.getElementById('radarImage');
                var oldWidth = radarImage.width;
                var oldHeight = radarImage.height;
                radarImage.src = newRadarImage.src;
                callback(oldWidth, oldHeight, width, height);
            }
            else { // loaded, but image is smaller than 10x10px (which is a placeholder error image)
                if (loadingAttempts < 5) { // try again for a maximum of 4 times
                    loadingAttempts++;
                    newRadarImage.src = sources(sourceName);
                }
                else { // after four failed attempts, show error message
                    GUI.errorLoading();
                }
            }
        };
        
        newRadarImage.onerror = function () {
            GUI.errorLoading();
        };
        
        newRadarImage.src = sources(sourceName);
    }
    
    function loadSnowRadar (callback) {
        var imageURLs = [],
            images = [],
            numberOfImagesToLoad = 12,
            numberOfImagesLoaded = 0,
            radarImage = document.getElementById('radarImage'),
            currentSnowImage = numberOfImagesToLoad - 1,
            i = 0;
        
        // Reverse loop
        for (i = numberOfImagesToLoad - 1; i >= 0; --i) {
            imageURLs[i] = "http://www1.buienradar.nl/forecast/sneeuwradar/sneeuwradar_" + i + ".gif?" + randomNumber();
            images[i] = new Image();
            images[i].onload = snowImageLoaded;
            
            // Set a different onload and onerror function for the first image.
            if (i === (numberOfImagesToLoad - 1)) {
                images[i].onload = function () {
                    width = this.width;
                    height = this.height;
                    if (width > 10 && height > 10) { // if the image is bigger than 10x10px
                        ++numberOfImagesLoaded;
                        var oldWidth = radarImage.width;
                        var oldHeight = radarImage.height;
                        radarImage.src = this.src;
                        callback(oldWidth, oldHeight, width, height);
                    }
                    else {
                        GUI.errorLoading();
                    }
                };
                images[i].onerror = function () {
                    GUI.errorLoading();
                };
            }
            
            images[i].src = imageURLs[i];
        }
        
        function snowImageLoaded() {
            ++numberOfImagesLoaded;
            if (numberOfImagesToLoad === numberOfImagesLoaded) {
                LoadingSpinner.stop();
                startSnowAnimation();
            }
        }
        
        function startSnowAnimation() {
            snowAnimationTimer = setInterval(animateSnowRadar, 800); // (function, milliseconds)
        }
        
        function animateSnowRadar() {
            --currentSnowImage;
            if (currentSnowImage < 0) {
                currentSnowImage = numberOfImagesToLoad - 1;
            }
            radarImage.src = images[currentSnowImage].src;
        }
    }
    
    function stopAnimation() {
        clearInterval(snowAnimationTimer);
    }
    
    function getWidth() {
        return width;
    }
    
    function getHeight() {
        return height;
    }
    
    // public methods
    return {
        loadImage: loadImage,
        loadSnowRadar: loadSnowRadar,
        stopAnimation: stopAnimation,
        getWidth: getWidth,
        getHeight: getHeight
    };
}());

var WetteronlineImage = (function () {
    // private properties
    var width, height;
    var loadingAttempts = 0;

    // private methods
    function timeToURL(selectString) {
        var currentDateTime = new Date();
        var currentMinutes = currentDateTime.getUTCMinutes();
        if (loadingAttempts > 0) {
            currentDateTime.setTime(currentDateTime - loadingAttempts * 900000); // go 15 min back in time
        }
        currentDateTime.setTime(currentDateTime - (currentMinutes % 15) * 60 * 1000); // only full increments of 5min
        var jaar = currentDateTime.getUTCFullYear(); // 4-digit year
        var dag = currentDateTime.getUTCDate().toTwoDigitString(); // 1..31
        var maand = (1 + currentDateTime.getUTCMonth()).toTwoDigitString(); // 0..11, 0=January
        var uur = currentDateTime.getUTCHours().toTwoDigitString(); // 0..23
        var minuut = currentDateTime.getUTCMinutes().toTwoDigitString(); // 0..59
        var urlTimeString = jaar + "/" + maand + "/" + dag + "/" + selectString + uur + minuut;
        return urlTimeString;
    }
    
    function sources(sourceName) {
        switch (sourceName) {
            case "mostRecent":
                return "http://www.wetteronline.de/daten/radar/ddlnw/" + timeToURL("") + ".gif";
            case "lastNinetyMinutes":
                return "http://www.wetteronline.de/daten/radar/ddlnw/" + timeToURL("std_") + ".gif";
            case "lastThreeHours":
                return "http://www.wetteronline.de/daten/radar/ddlnw/" + timeToURL("vie_") + ".gif";
        }
    }
    
    function loadImage(sourceName, callback) {
        var newRadarImage = new Image();
        newRadarImage.onload = function () {
            LoadingSpinner.stop();
            loadingAttempts = 0;
            width = newRadarImage.width;
            height = newRadarImage.height;
            var radarImage = document.getElementById('radarImage');
            var oldWidth = radarImage.width;
            var oldHeight = radarImage.height;
            radarImage.src = newRadarImage.src;
            callback(oldWidth, oldHeight, width, height);
        };
        newRadarImage.onerror = function () {
            if (loadingAttempts < 4) { // retry three times
                loadingAttempts++;
                newRadarImage.src = sources(sourceName);
            }
            else { // after three failed attempts, show error message
                GUI.errorLoading();
            }
        };
        newRadarImage.src = sources(sourceName);
    }
    
    function getWidth() {
        return width;
    }
    
    function getHeight() {
        return height;
    }

    // public methods
    return {
        loadImage: loadImage,
        getWidth: getWidth,
        getHeight: getHeight
    };
}());

var Sat24Image = (function () {
    // private properties
    var width, height;
    
    // private methods
    function randomNumber() {
        var returnValue = "",
            i = 0;
        for (i = 0; i < 6; i++) {
            returnValue += Math.round(10*Math.random());
        }
        return returnValue;
    }
    
    function sources(sourceName) {
        switch (sourceName) {
            case "lastVis":
                return "http://www.sat24.com/images.php?country=nl&type=last&sat=vis&rnd=" + randomNumber();
            case "loopVis":
                return "http://www.sat24.com/images.php?country=nl&sat=vis&rnd=" + randomNumber();
            case "lastIR":
                return "http://www.sat24.com/images.php?country=nl&type=last&sat=ir&rnd=" + randomNumber();
            case "loopIR":
                return "http://www.sat24.com/images.php?country=nl&sat=ir&rnd=" + randomNumber();
        }
    }
    
    function loadImage(sourceName, callback) {
        var newRadarImage = new Image();
        
        newRadarImage.onload = function () {
            width = newRadarImage.width;
            height = newRadarImage.height;
            var radarImage = document.getElementById('radarImage');
            var oldWidth = radarImage.width;
            var oldHeight = radarImage.height;
            radarImage.src = newRadarImage.src;
            LoadingSpinner.stop();
            callback(oldWidth, oldHeight, width, height);
        };
        
        newRadarImage.onerror = function () {
            GUI.errorLoading();
        };
        
        newRadarImage.src = sources(sourceName);
    }
    
    function getWidth() {
        return width;
    }
    
    function getHeight() {
        return height;
    }
    
    // public methods
    return {
        loadImage: loadImage,
        getWidth: getWidth,
        getHeight: getHeight
    };
}());

var KnmiImage = (function () {
    // private properties
    var width, height;
    
    function randomNumber() {
        var returnValue = "",
            i = 0;
        for (i = 0; i < 6; i++) {
            returnValue += Math.round(10*Math.random());
        }
        return returnValue;
    }
    
    function sources(sourceName) {
        switch (sourceName) {
            case "temp":
                return "http://cdn.knmi.nl/knmi/map/page/weer/actueel-weer/temperatuur.png?" + randomNumber();
            // case "windchill":
            //     return "http://www.knmi.nl/actueel/images/windchillgmt.png?rnd=" + randomNumber();
            case "zicht":
                return "http://cdn.knmi.nl/knmi/map/page/weer/actueel-weer/zicht.png?" + randomNumber();
            case "windMsec":
                return "http://cdn.knmi.nl/knmi/map/page/weer/actueel-weer/windsnelheid.png?" + randomNumber();
            case "windBeaufort":
                return "http://cdn.knmi.nl/knmi/map/page/weer/actueel-weer/windkracht.png?" + randomNumber();
            case "neerslagradar":
                return "http://cdn.knmi.nl/knmi/map/page/weer/actueel-weer/neerslagradar/WWWRADAR_loop.gif?" + randomNumber();
        }
    }

    function loadImage(sourceName, callback) {
        var newRadarImage = new Image();
        
        newRadarImage.onload = function () {
            width = newRadarImage.width;
            height = newRadarImage.height;
            var radarImage = document.getElementById('radarImage');
            var oldWidth = radarImage.width;
            var oldHeight = radarImage.height;
            radarImage.src = newRadarImage.src;
            LoadingSpinner.stop();
            callback(oldWidth, oldHeight, width, height);
        };
        
        newRadarImage.onerror = function () {
            GUI.errorLoading();
        };
        
        newRadarImage.src = sources(sourceName);
    }
    
    function getWidth() {
        return width;
    }
    
    function getHeight() {
        return height;
    }
    
    // public methods
    return {
        loadImage: loadImage,
        getWidth: getWidth,
        getHeight: getHeight
    };
}());